@@ -1,5 +1,13 @@
 #include "obs-module.h"
 #include "version.h"
+#include <util/platform.h>
+#include <util/dstr.h>
+#include <obs-frontend-api.h>
+
+#ifdef _WIN32
+#include <windows.h>
+#include <shellapi.h>
+#endif
 
 #define LOG_OFFSET_DB 6.0f
 #define LOG_RANGE_DB 96.0f
@@ -11,7 +19,7 @@ struct scene_as_transition {
 	bool transitioning;
 	float transition_point;
 	float duration;
-	const char *filter_name;
+	char *filter_name;
 
 	obs_transition_audio_mix_callback_t mix_a;
 	obs_transition_audio_mix_callback_t mix_b;
@@ -22,7 +30,7 @@ struct scene_as_transition {
 static const char *scene_as_transition_get_name(void *type_data)
 {
 	UNUSED_PARAMETER(type_data);
-	return obs_module_text("Scene");
+	return obs_module_text("Plugin.Name");
 }
 
 static inline float calc_fade(float t, float mul)
@@ -58,6 +66,8 @@ static float mix_b_cross_fade(void *data, float t)
 void scene_as_transition_update(void *data, obs_data_t *settings)
 {
 	struct scene_as_transition *st = data;
+	if (!st)
+		return;
 
 	obs_source_release(st->transition_scene);
 	st->transition_scene =
@@ -82,11 +92,43 @@ void scene_as_transition_update(void *data, obs_data_t *settings)
 
 	const char *filter_name = obs_data_get_string(settings, "filter");
 
-	if (st->filter)
-		obs_source_release(st->filter);
+	// Check if filter name has changed to avoid unnecessary re-fetching
+	bool filter_name_changed = !st->filter_name ||
+				   strcmp(st->filter_name, filter_name) != 0;
 
-	st->filter = obs_source_get_filter_by_name(st->transition_scene,
-						   filter_name);
+	// Store filter name for lazy loading
+	if (filter_name_changed) {
+		if (st->filter_name)
+			bfree(st->filter_name);
+		st->filter_name = bstrdup(filter_name);
+
+		// Release existing filter only if name changed
+		if (st->filter) {
+			obs_source_release(st->filter);
+			st->filter = NULL;
+		}
+
+		// Check if a valid filter is selected (not "NoFilterSelected" or empty)
+		const char *no_filter_text = obs_module_text("Filter.NoSelection");
+		bool has_valid_filter = filter_name && *filter_name &&
+					strcmp(filter_name, no_filter_text) != 0 &&
+					strcmp(filter_name, "filter") != 0;
+
+		if (has_valid_filter && st->transition_scene) {
+			st->filter = obs_source_get_filter_by_name(st->transition_scene,
+								   filter_name);
+			if (!st->filter) {
+				blog(LOG_WARNING,
+				     "[StreamUP Scene as Transition] Failed to find filter '%s' on scene '%s'. "
+				     "Filter may not be loaded yet and will be retried during transition.",
+				     filter_name, obs_source_get_name(st->transition_scene));
+			} else {
+				blog(LOG_INFO,
+				     "[StreamUP Scene as Transition] Successfully loaded filter '%s' from scene '%s'",
+				     filter_name, obs_source_get_name(st->transition_scene));
+			}
+		}
+	}
 
 	st->transition_a_mul = (1.0f / st->transition_point);
 	st->transition_b_mul = (1.0f / (1.0f - st->transition_point));
@@ -148,6 +190,10 @@ static void scene_as_transition_destroy(void *data)
 {
 	struct scene_as_transition *st = data;
 	obs_source_release(st->transition_scene);
+	if (st->filter)
+		obs_source_release(st->filter);
+	if (st->filter_name)
+		bfree(st->filter_name);
 	bfree(data);
 }
 
@@ -155,6 +201,11 @@ static void scene_as_transition_video_render(void *data, gs_effect_t *effect)
 {
 	struct scene_as_transition *st = data;
 
+	// NULL safety check
+	if (!st || !st->transition_scene) {
+		return;
+	}
+
 	float t = obs_transition_get_time(st->source);
 	bool use_a = t < st->transition_point;
 
@@ -175,6 +226,33 @@ static void scene_as_transition_video_render(void *data, gs_effect_t *effect)
 				obs_source_inc_showing(st->transition_scene);
 			if (obs_source_active(st->source))
 				obs_source_inc_active(st->transition_scene);
+
+			// Lazy load filter if it wasn't available during init
+			if (!st->filter && st->filter_name && st->transition_scene) {
+				const char *no_filter_text = obs_module_text("Filter.NoSelection");
+				bool has_valid_filter = st->filter_name && *st->filter_name &&
+							strcmp(st->filter_name, no_filter_text) != 0 &&
+							strcmp(st->filter_name, "filter") != 0;
+
+				if (has_valid_filter) {
+					st->filter = obs_source_get_filter_by_name(
+						st->transition_scene, st->filter_name);
+					if (st->filter) {
+						blog(LOG_INFO,
+						     "[StreamUP Scene as Transition] Lazy loading succeeded: "
+						     "Found filter '%s' on scene '%s'",
+						     st->filter_name,
+						     obs_source_get_name(st->transition_scene));
+					} else {
+						blog(LOG_WARNING,
+						     "[StreamUP Scene as Transition] Lazy loading failed: "
+						     "Filter '%s' still not found on scene '%s'",
+						     st->filter_name,
+						     obs_source_get_name(st->transition_scene));
+					}
+				}
+			}
+
 			if (st->filter)
 				obs_source_set_enabled(st->filter, true);
 		}
@@ -185,6 +263,10 @@ static void scene_as_transition_video_render(void *data, gs_effect_t *effect)
 			obs_source_dec_active(st->transition_scene);
 		if (obs_source_showing(st->source))
 			obs_source_dec_showing(st->transition_scene);
+
+		// Disable filter when transition ends
+		if (st->filter)
+			obs_source_set_enabled(st->filter, false);
 	}
 
 	UNUSED_PARAMETER(effect);
@@ -196,7 +278,7 @@ static bool scene_as_transition_audio_render(void *data, uint64_t *ts_out,
 					     size_t sample_rate)
 {
 	struct scene_as_transition *st = data;
-	if (!st)
+	if (!st || !st->transition_scene)
 		return false;
 
 	uint64_t ts = 0;
@@ -251,7 +333,7 @@ void scene_as_transition_defaults(obs_data_t *settings)
 	obs_data_set_default_double(settings, "transition_point", 50.0);
 	obs_data_set_default_double(settings, "transition_point_ms", 500.0);
 	obs_data_set_default_string(settings, "filter",
-				    obs_module_text("NoFilterSelected"));
+				    obs_module_text("Filter.NoSelection"));
 	obs_data_set_default_string(settings, "prev_scene", "");
 	obs_data_set_default_double(settings, "audio_volume", 100.0);
 }
@@ -309,12 +391,12 @@ static bool scene_modified(obs_properties_t *props, obs_property_t *property,
 
 		obs_property_list_clear(filter);
 		obs_property_list_add_string(
-			filter, obs_module_text("NoFilterSelected"), "filter");
+			filter, obs_module_text("Filter.NoSelection"), "filter");
 		obs_source_enum_filters(
 			scene, scene_as_transition_list_add_filter, filter);
 
 		obs_data_set_string(settings, "filter",
-				    obs_module_text("NoFilterSelected"));
+				    obs_module_text("Filter.NoSelection"));
 		obs_data_set_string(settings, "prev_scene", scene_name);
 
 		obs_source_release(scene);
@@ -349,81 +431,81 @@ obs_properties_t *scene_as_transition_properties(void *data)
 	obs_properties_t *props = obs_properties_create();
 
 	obs_property_t *scene = obs_properties_add_list(
-		props, "scene", obs_module_text("Scene"),
+		props, "scene", obs_module_text("Scene.Name"),
 		OBS_COMBO_TYPE_EDITABLE, OBS_COMBO_FORMAT_STRING);
 	obs_property_set_long_description(scene,
-					  obs_module_text("SceneDescription"));
+					  obs_module_text("Scene.Description"));
 	obs_enum_scenes(scene_as_transition_list_add_scene, scene);
 	obs_property_set_modified_callback(scene, scene_modified);
 
 	obs_property_t *p = obs_properties_add_float(
-		props, "duration", obs_module_text("Duration"), 0.0, 30000.0,
+		props, "duration", obs_module_text("Transition.Duration"), 0.0, 30000.0,
 		100.0);
 	obs_property_float_set_suffix(p, " ms");
 	obs_property_set_long_description(
-		p, obs_module_text("DurationDescription"));
+		p, obs_module_text("Transition.Duration.Description"));
 
 	obs_properties_t *transition_point_group = obs_properties_create();
 
 	obs_properties_add_group(props, "transition_point_group",
-				 obs_module_text("TransitionPointSettings"),
+				 obs_module_text("TransitionPoint.Settings"),
 				 OBS_GROUP_NORMAL, transition_point_group);
 
 	p = obs_properties_add_list(transition_point_group, "tp_type",
-				    obs_module_text("TransitionPointType"),
+				    obs_module_text("TransitionPoint.Type"),
 				    OBS_COMBO_TYPE_LIST, OBS_COMBO_FORMAT_INT);
 	obs_property_list_add_int(
-		p, obs_module_text("TransitionPointPercentage"), 0);
-	obs_property_list_add_int(p, obs_module_text("TransitionPointTime"), 1);
+		p, obs_module_text("TransitionPoint.Percentage"), 0);
+	obs_property_list_add_int(p, obs_module_text("TransitionPoint.Time"), 1);
 	obs_property_set_long_description(
-		p, obs_module_text("TransitionPointTypeDescription"));
+		p, obs_module_text("TransitionPoint.Type.Description"));
 	obs_property_set_modified_callback(p, transition_point_type_modified);
 	p = obs_properties_add_float_slider(transition_point_group,
 					    "transition_point",
-					    obs_module_text("TransitionPoint"),
+					    obs_module_text("TransitionPoint.Name"),
 					    0, 100.0, 1.0);
 	obs_property_float_set_suffix(p, "%");
 	obs_property_set_long_description(
-		p, obs_module_text("TransitionPointPercentageDescription"));
+		p, obs_module_text("TransitionPoint.Percentage.Description"));
 
 	p = obs_properties_add_float(transition_point_group,
 				     "transition_point_ms",
-				     obs_module_text("TransitionPoint"), 0,
+				     obs_module_text("TransitionPoint.Name"), 0,
 				     30000.0, 100.0);
 	obs_property_float_set_suffix(p, " ms");
 	obs_property_set_long_description(
-		p, obs_module_text("TransitionPointTimeDescription"));
+		p, obs_module_text("TransitionPoint.Time.Description"));
 
 	obs_properties_t *audio_group = obs_properties_create();
 
 	obs_properties_add_group(props, "audio_group",
-				 obs_module_text("AudioSettings"),
+				 obs_module_text("Audio.Settings"),
 				 OBS_GROUP_NORMAL, audio_group);
 	obs_property_t *audio_fade_style = obs_properties_add_list(
 		audio_group, "audio_fade_style",
-		obs_module_text("AudioFadeStyle"), OBS_COMBO_TYPE_LIST,
+		obs_module_text("Audio.FadeStyle"), OBS_COMBO_TYPE_LIST,
 		OBS_COMBO_FORMAT_INT);
 	obs_property_set_long_description(
-		audio_fade_style, obs_module_text("AudioFadeStyleDescription"));
+		audio_fade_style, obs_module_text("Audio.FadeStyle.Description"));
 	obs_property_list_add_int(audio_fade_style,
-				  obs_module_text("FadeOutFadeIn"), 0);
+				  obs_module_text("Audio.FadeStyle.FadeOutIn"), 0);
 	obs_property_list_add_int(audio_fade_style,
-				  obs_module_text("CrossFade"), 1);
+				  obs_module_text("Audio.FadeStyle.CrossFade"), 1);
 
 	p = obs_properties_add_float_slider(audio_group, "audio_volume",
-					    obs_module_text("AudioVolume"), 0,
+					    obs_module_text("Audio.Volume"), 0,
 					    100.0, 1.0);
 	obs_property_float_set_suffix(p, "%");
 	obs_property_set_long_description(
-		p, obs_module_text("AudioVolumeDescription"));
+		p, obs_module_text("Audio.Volume.Description"));
 
 	obs_property_t *filter = obs_properties_add_list(
-		props, "filter", obs_module_text("FilterToTrigger"),
+		props, "filter", obs_module_text("Filter.ToTrigger"),
 		OBS_COMBO_TYPE_EDITABLE, OBS_COMBO_FORMAT_STRING);
 	obs_property_list_add_string(
-		filter, obs_module_text("NoFilterSelected"), "filter");
+		filter, obs_module_text("Filter.NoSelection"), "filter");
 	obs_property_set_long_description(
-		filter, obs_module_text("FilterToTriggerDescription"));
+		filter, obs_module_text("Filter.ToTrigger.Description"));
 	obs_source_enum_filters(st->transition_scene,
 				scene_as_transition_list_add_filter, filter);
 
@@ -460,18 +542,232 @@ OBS_MODULE_AUTHOR("Andi Stone")
 OBS_MODULE_USE_DEFAULT_LOCALE("scene-as-transition", "en-US")
 MODULE_EXPORT const char *obs_module_description(void)
 {
-	return obs_module_text("Description");
+	return obs_module_text("Plugin.Description");
 }
 
 MODULE_EXPORT const char *obs_module_name(void)
 {
-	return obs_module_text("SceneAsTransition");
+	return obs_module_text("Plugin.Name");
+}
+
+static void open_folder_and_highlight(const char *file_path)
+{
+#ifdef _WIN32
+	// Convert forward slashes to backslashes for Windows
+	struct dstr windows_path = {0};
+	dstr_copy(&windows_path, file_path);
+	dstr_replace(&windows_path, "/", "\\");
+
+	// Build the explorer command to select the file
+	struct dstr command = {0};
+	dstr_printf(&command, "/select,\"%s\"", windows_path.array);
+
+	// Open explorer and highlight the file
+	ShellExecuteA(NULL, "open", "explorer.exe", command.array, NULL,
+		      SW_SHOWNORMAL);
+
+	dstr_free(&command);
+	dstr_free(&windows_path);
+#elif __APPLE__
+	// macOS: Use 'open -R' to reveal in Finder
+	struct dstr command = {0};
+	dstr_printf(&command, "open -R \"%s\"", file_path);
+	int result = system(command.array);
+	if (result != 0) {
+		blog(LOG_WARNING,
+		     "[StreamUP Scene as Transition] Failed to open Finder");
+	}
+	dstr_free(&command);
+#else
+	// Linux: Open the parent directory
+	struct dstr dir_path = {0};
+	dstr_copy(&dir_path, file_path);
+	char *last_slash = strrchr(dir_path.array, '/');
+	if (last_slash) {
+		*last_slash = '\0';
+		struct dstr command = {0};
+		dstr_printf(&command, "xdg-open \"%s\"", dir_path.array);
+		int result = system(command.array);
+		if (result != 0) {
+			blog(LOG_WARNING,
+			     "[StreamUP Scene as Transition] Failed to open file manager");
+		}
+		dstr_free(&command);
+	}
+	dstr_free(&dir_path);
+#endif
+}
+
+struct old_plugin_check_data {
+	char *old_plugin_path;
+};
+
+static void show_old_plugin_dialog(void *data)
+{
+	struct old_plugin_check_data *check_data = data;
+	if (!check_data || !check_data->old_plugin_path)
+		return;
+
+#ifdef _WIN32
+	// Create message for the dialog
+	struct dstr message = {0};
+	dstr_printf(&message,
+		    "An old version of Scene as Transition has been detected:\n\n"
+		    "%s\n\n"
+		    "This old version may cause conflicts with the new StreamUP Scene as Transition plugin.\n\n"
+		    "Would you like to open the plugins folder to remove it?",
+		    check_data->old_plugin_path);
+
+	// Show message box with Yes/No buttons
+	int result = MessageBoxA(NULL, message.array,
+				 "StreamUP Scene as Transition - Old Plugin Detected",
+				 MB_YESNO | MB_ICONWARNING | MB_TOPMOST);
+
+	if (result == IDYES) {
+		open_folder_and_highlight(check_data->old_plugin_path);
+	}
+
+	dstr_free(&message);
+#else
+	// For non-Windows, just open the folder
+	open_folder_and_highlight(check_data->old_plugin_path);
+#endif
+
+	// Clean up
+	bfree(check_data->old_plugin_path);
+	bfree(check_data);
+}
+
+static void find_old_plugin_files(struct dstr *found_path)
+{
+	// Get the binary module path
+	const char *bin_path = obs_get_module_binary_path(obs_current_module());
+	if (!bin_path) {
+		blog(LOG_INFO,
+		     "[StreamUP Scene as Transition] Unable to get module binary path");
+		return;
+	}
+
+	blog(LOG_INFO,
+	     "[StreamUP Scene as Transition] Current module binary path: %s",
+	     bin_path);
+
+	struct dstr plugin_dir = {0};
+	dstr_init_copy(&plugin_dir, bin_path);
+
+	// Navigate to the plugins directory
+	char *last_slash = strrchr(plugin_dir.array, '/');
+	if (!last_slash)
+		last_slash = strrchr(plugin_dir.array, '\\');
+
+	if (!last_slash) {
+		blog(LOG_INFO,
+		     "[StreamUP Scene as Transition] Unable to determine plugin directory");
+		dstr_free(&plugin_dir);
+		return;
+	}
+
+	// Truncate to directory
+	dstr_resize(&plugin_dir, last_slash - plugin_dir.array + 1);
+
+	blog(LOG_INFO,
+	     "[StreamUP Scene as Transition] Checking for old plugin in: %s",
+	     plugin_dir.array);
+
+	// List of possible old plugin names to check
+	const char *old_plugin_names[] = {"scene-as-transition.dll",
+					  "obs-scene-as-transition.dll",
+					  "SceneAsTransition.dll", NULL};
+
+	struct dstr old_plugin_path = {0};
+
+	for (int i = 0; old_plugin_names[i] != NULL; i++) {
+		dstr_copy(&old_plugin_path, plugin_dir.array);
+		dstr_cat(&old_plugin_path, old_plugin_names[i]);
+
+		blog(LOG_INFO,
+		     "[StreamUP Scene as Transition] Checking for: %s",
+		     old_plugin_path.array);
+
+		if (os_file_exists(old_plugin_path.array)) {
+			dstr_copy(found_path, old_plugin_path.array);
+			blog(LOG_INFO,
+			     "[StreamUP Scene as Transition] Found old plugin file at: %s",
+			     old_plugin_path.array);
+			dstr_free(&old_plugin_path);
+			dstr_free(&plugin_dir);
+			return;
+		}
+	}
+
+	blog(LOG_INFO,
+	     "[StreamUP Scene as Transition] No old plugin file found in binary directory");
+
+	dstr_free(&old_plugin_path);
+	dstr_free(&plugin_dir);
+}
+
+static void check_for_old_plugin(void)
+{
+	// First check if the source is already registered (indicates old plugin is loaded)
+	const char *source_id = "scene_as_transition";
+	bool source_already_exists = false;
+
+	// Try to get the source output flags - if the source type is registered, this returns non-zero
+	uint32_t flags = obs_get_source_output_flags(source_id);
+	if (flags != 0) {
+		source_already_exists = true;
+		blog(LOG_WARNING,
+		     "[StreamUP Scene as Transition] =========================================");
+		blog(LOG_WARNING,
+		     "[StreamUP Scene as Transition] OLD PLUGIN DETECTED!");
+		blog(LOG_WARNING,
+		     "[StreamUP Scene as Transition] The source ID '%s' is already registered.",
+		     source_id);
+		blog(LOG_WARNING,
+		     "[StreamUP Scene as Transition] This indicates the old 'Scene As Transition' plugin");
+		blog(LOG_WARNING,
+		     "[StreamUP Scene as Transition] is currently loaded and must be removed.");
+		blog(LOG_WARNING,
+		     "[StreamUP Scene as Transition] =========================================");
+	}
+
+	// Try to find the actual file
+	struct dstr old_plugin_path = {0};
+	find_old_plugin_files(&old_plugin_path);
+
+	// Show dialog if we detected the old plugin
+	if (source_already_exists) {
+		struct old_plugin_check_data *check_data =
+			bzalloc(sizeof(struct old_plugin_check_data));
+
+		// Use the found file path if available, otherwise just indicate we found it
+		if (!dstr_is_empty(&old_plugin_path)) {
+			check_data->old_plugin_path =
+				bstrdup(old_plugin_path.array);
+		} else {
+			// We know it's loaded but couldn't find the file
+			// Provide a generic message
+			check_data->old_plugin_path = bstrdup(
+				"Old plugin is loaded but file location could not be determined");
+		}
+
+		// Queue task to show dialog on UI thread
+		obs_queue_task(OBS_TASK_UI, show_old_plugin_dialog, check_data,
+			       false);
+	}
+
+	dstr_free(&old_plugin_path);
 }
 
 bool obs_module_load(void)
 {
-	blog(LOG_INFO, "[Scene As Transition] loaded version %s",
+	blog(LOG_INFO, "[StreamUP Scene as Transition] loaded version %s",
 	     PROJECT_VERSION);
+
+	// Check for old plugin version
+	check_for_old_plugin();
+
 	obs_register_source(&scene_as_transition);
 	return true;
 }
